package nccloud.web.gpmc.gpm.contract.action;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import nc.bs.logging.Logger;
import nc.itf.gpmc.gpm.IGpmcguacontractQueryService;
import nc.vo.bd.bankdoc.BankdocVO;
import nc.vo.gpmc.gpm.contract.AggContractVersionVO;
import nc.vo.gpmc.gpm.contract.AggGuaContractVO;
import nc.vo.ml.NCLangRes4VoTransl;
import nc.vo.pub.BusinessException;
import nc.vo.pub.ISuperVO;
import nc.vo.tmpub.util.ArrayUtil;
import nc.vo.tmpub.util.StringUtil;
import nccloud.dto.baseapp.querytree.dataformat.QueryTreeFormatVO;
import nccloud.framework.core.exception.ExceptionUtils;
import nccloud.framework.core.json.IJson;
import nccloud.framework.service.ServiceLocator;
import nccloud.framework.web.container.IRequest;
import nccloud.framework.web.convert.translate.Translator;
import nccloud.framework.web.json.JsonFactory;
import nccloud.framework.web.processor.template.GridConvertProcessor;
import nccloud.framework.web.ui.model.PageInfo;
import nccloud.framework.web.ui.pattern.grid.Grid;
import nccloud.web.common.gpmc.action.AbstractAction;
import nccloud.web.gpmc.gpm.pub.ListQryResult;
import nccloud.web.tmpub.pub.translate.ExtraTranslator;

public class GuaContractQueryVersionListAction extends AbstractAction<AggGuaContractVO> {
	public Object doAction(IRequest request) {
		String read = request.read();
		IJson json = JsonFactory.create();
		QueryTreeFormatVO info = (QueryTreeFormatVO) json.fromJson(read, QueryTreeFormatVO.class);
		if (StringUtil.isNull(info.getPageInfo())) {
			ExceptionUtils.wrapBusinessException(
					NCLangRes4VoTransl.getNCLangRes().getStrByID("gpmc_ncc003_0", "0gpmc_ncc003-0014"));
		}

		String pageCode = info.getPageCode();
		if (StringUtil.isNull(pageCode)) {
			ExceptionUtils.wrapBusinessException(
					NCLangRes4VoTransl.getNCLangRes().getStrByID("gpmc_ncc003_0", "0gpmc_ncc003-0015"));
		}

		try {
			String originalPk = info.getDef1();
			AggContractVersionVO[] vos = ((IGpmcguacontractQueryService) ServiceLocator
					.find(IGpmcguacontractQueryService.class)).queryVersionsByOrigPk(originalPk);
			if (vos != null && vos.length >= 1) {
				Map<String, Object> tmGrid = this.buildQryResult(vos);
				return tmGrid;
			} else {
				throw new BusinessException(
						NCLangRes4VoTransl.getNCLangRes().getStrByID("gpmc_ncc003_0", "0gpmc_ncc003-0016"));
			}
		} catch (BusinessException var9) {
			Logger.error(var9.getMessage(), var9);
			ExceptionUtils.wrapException(var9);
			return null;
		}
	}

	public ListQryResult<AggContractVersionVO> query(String originalPk) throws BusinessException {
		ListQryResult<AggContractVersionVO> result = new ListQryResult();
		AggContractVersionVO[] vos = ((IGpmcguacontractQueryService) ServiceLocator
				.find(IGpmcguacontractQueryService.class)).queryVersionsByOrigPk(originalPk);
		if (vos != null && vos.length >= 1) {
			result.setAggVOs(vos);
			return result;
		} else {
			throw new BusinessException(
					NCLangRes4VoTransl.getNCLangRes().getStrByID("gpmc_ncc003_0", "0gpmc_ncc003-0016"));
		}
	}

	private Map<String, Object> buildQryResult(AggContractVersionVO[] vos) throws BusinessException {
		Map<String, Object> result = new HashMap();
		Map<String, Object> data = new HashMap();
		List<Map<String, Object>> rows = new ArrayList();
		AggContractVersionVO[] var5 = vos;
		int var6 = vos.length;

		for (int var7 = 0; var7 < var6; ++var7) {
			AggContractVersionVO vo = var5[var7];
			Map<String, Object> row = new HashMap();
			row.put("isleaf", true);
			row.put("refcode", vo.getParentVO().getVbillno());
			row.put("refname", NCLangRes4VoTransl.getNCLangRes().getStrByID("gpmc_ncc003_0", "0gpmc_ncc003-0017")
					+ vo.getParentVO().getVersionno());
			row.put("refpk", vo.getParentVO().getPk_version());
			rows.add(row);
		}

		data.put("rows", rows);
		result.put("data", data);
		result.put("error", (Object) null);
		result.put("success", true);
		return result;
	}

	private PageInfo buildPageInfo(ListQryResult<AggContractVersionVO> result) throws BusinessException {
		PageInfo pageInfo = new PageInfo();
		pageInfo.setTotal(result.getTotalNum());
		pageInfo.setPageSize(result.getPageSize());
		pageInfo.setPageIndex(result.getPageIndex());
		pageInfo.setTotalPage(result.getTotalPageNum());
		return pageInfo;
	}

	private Grid buildGridData(ListQryResult<AggContractVersionVO> result, PageInfo pageInfo, String pageCode)
			throws BusinessException {
		AggContractVersionVO[] vos = (AggContractVersionVO[]) result.getAggVOs();
		if (ArrayUtil.isNull(vos)) {
			return null;
		} else {
			Grid grid = this.doReturn(vos, pageCode);
			grid.getModel().setAllpks(result.getAllPKs());
			grid.getModel().setPageinfo(pageInfo);
			return grid;
		}
	}

	private Grid doReturn(AggContractVersionVO[] vos, String pageCode) throws BusinessException {
		GridConvertProcessor processor = new GridConvertProcessor();
		Grid grid = processor.convert(pageCode, this.fetchHeadVOs(vos));
		Translator translator = new Translator();
		translator.translate(grid);
		ExtraTranslator addExtraTransField = new ExtraTranslator();
		addExtraTransField.addExtraTransField("fincreditor", "name", BankdocVO.class);
		addExtraTransField.addExtraTransField("finguarantor", "name", BankdocVO.class);
		addExtraTransField.addExtraTransField("warrantorfin", "name", BankdocVO.class);
		addExtraTransField.translate(grid);
		return grid;
	}

	private ISuperVO[] fetchHeadVOs(AggContractVersionVO[] vos) throws BusinessException {
		if (ArrayUtil.isNull(vos)) {
			return null;
		} else {
			List<ISuperVO> headVOList = new ArrayList();
			AggContractVersionVO[] var3 = vos;
			int var4 = vos.length;

			for (int var5 = 0; var5 < var4; ++var5) {
				AggContractVersionVO aggVO = var3[var5];
				if (aggVO != null && aggVO.getParent() != null) {
					headVOList.add(aggVO.getParent());
				}
			}

			if (headVOList.isEmpty()) {
				return null;
			} else {
				return (ISuperVO[]) headVOList.toArray(new ISuperVO[headVOList.size()]);
			}
		}
	}
}